@@ -16,20 +16,31 @@
 ** command of TCL to do a lot of this, but there are some issues
 ** with historical versions of the "binary" command.  So it seems
 ** easier and safer to build our own mechanism.
+**
+** $Id: test_hexio.c,v 1.1 2007/04/06 15:02:14 drh Exp $
 */
-#include "sqliteInt.h"
 #include "tcl.h"
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 
+/*
+hexio_read  filename offset amt
+hexio_write filename offset hexdata
+hexio_get_int  hexdata
+hexio_get_varint  hexdata
+hexio_render_int8  integer
+hexio_render_int16  integer
+hexio_render_int38  integer
+hexio_render_varint  integer
+*/
 
 /*
 ** Convert binary to hex.  The input zBuf[] contains N bytes of
 ** binary data.  zBuf[] is 2*n+1 bytes long.  Overwrite zBuf[]
 ** with a hexadecimal representation of its original binary input.
 */
-void sqlite3TestBinToHex(unsigned char *zBuf, int N){
+static void binToHex(unsigned char *zBuf, int N){
   const unsigned char zHex[] = "0123456789ABCDEF";
   int i, j;
   unsigned char c;
@@ -49,7 +60,7 @@ void sqlite3TestBinToHex(unsigned char *zBuf, int N){
 ** the binary data.  Spaces in the original input are ignored.
 ** Return the number of bytes of binary rendered.
 */
-int sqlite3TestHexToBin(const unsigned char *zIn, int N, unsigned char *aOut){
+static int hexToBin(const unsigned char *zIn, int N, unsigned char *aOut){
   const unsigned char aMap[] = {
      0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
      0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0,
@@ -113,27 +124,24 @@ static int hexio_read(
   if( Tcl_GetIntFromObj(interp, objv[2], &offset) ) return TCL_ERROR;
   if( Tcl_GetIntFromObj(interp, objv[3], &amt) ) return TCL_ERROR;
   zFile = Tcl_GetString(objv[1]);
-  zBuf = sqlite3_malloc( amt*2+1 );
+  zBuf = malloc( amt*2+1 );
   if( zBuf==0 ){
     return TCL_ERROR;
   }
-  in = fopen(zFile, "rb");
-  if( in==0 ){
-    in = fopen(zFile, "r");
-  }
+  in = fopen(zFile, "r");
   if( in==0 ){
     Tcl_AppendResult(interp, "cannot open input file ", zFile, 0);
     return TCL_ERROR;
   }
   fseek(in, offset, SEEK_SET);
-  got = (int)fread(zBuf, 1, amt, in);
+  got = fread(zBuf, 1, amt, in);
   fclose(in);
   if( got<0 ){
     got = 0;
   }
-  sqlite3TestBinToHex(zBuf, got);
+  binToHex(zBuf, got);
   Tcl_AppendResult(interp, zBuf, 0);
-  sqlite3_free(zBuf);
+  free(zBuf);
   return TCL_OK;
 }
 
@@ -164,22 +172,19 @@ static int hexio_write(
   if( Tcl_GetIntFromObj(interp, objv[2], &offset) ) return TCL_ERROR;
   zFile = Tcl_GetString(objv[1]);
   zIn = (const unsigned char *)Tcl_GetStringFromObj(objv[3], &nIn);
-  aOut = sqlite3_malloc( nIn/2 );
+  aOut = malloc( nIn/2 );
   if( aOut==0 ){
     return TCL_ERROR;
   }
-  nOut = sqlite3TestHexToBin(zIn, nIn, aOut);
-  out = fopen(zFile, "r+b");
-  if( out==0 ){
-    out = fopen(zFile, "r+");
-  }
+  nOut = hexToBin(zIn, nIn, aOut);
+  out = fopen(zFile, "r+");
   if( out==0 ){
     Tcl_AppendResult(interp, "cannot open output file ", zFile, 0);
     return TCL_ERROR;
   }
   fseek(out, offset, SEEK_SET);
-  written = (int)fwrite(aOut, 1, nOut, out);
-  sqlite3_free(aOut);
+  written = fwrite(aOut, 1, nOut, out);
+  free(aOut);
   fclose(out);
   Tcl_SetObjResult(interp, Tcl_NewIntObj(written));
   return TCL_OK;
@@ -209,18 +214,18 @@ static int hexio_get_int(
     return TCL_ERROR;
   }
   zIn = (const unsigned char *)Tcl_GetStringFromObj(objv[1], &nIn);
-  aOut = sqlite3_malloc( nIn/2 );
+  aOut = malloc( nIn/2 );
   if( aOut==0 ){
     return TCL_ERROR;
   }
-  nOut = sqlite3TestHexToBin(zIn, nIn, aOut);
+  nOut = hexToBin(zIn, nIn, aOut);
   if( nOut>=4 ){
     memcpy(aNum, aOut, 4);
   }else{
     memset(aNum, 0, sizeof(aNum));
-    memcpy(&aNum[4-nOut], aOut, nOut);
+    memcpy(&aNum[4-nOut], aOut, 4-nOut);
   }
-  sqlite3_free(aOut);
+  free(aOut);
   val = (aNum[0]<<24) | (aNum[1]<<16) | (aNum[2]<<8) | aNum[3];
   Tcl_SetObjResult(interp, Tcl_NewIntObj(val));
   return TCL_OK;
@@ -248,7 +253,7 @@ static int hexio_render_int16(
   if( Tcl_GetIntFromObj(interp, objv[1], &val) ) return TCL_ERROR;
   aNum[0] = val>>8;
   aNum[1] = val;
-  sqlite3TestBinToHex(aNum, 2);
+  binToHex(aNum, 2);
   Tcl_SetObjResult(interp, Tcl_NewStringObj((char*)aNum, 4));
   return TCL_OK;
 }
@@ -277,91 +282,11 @@ static int hexio_render_int32(
   aNum[1] = val>>16;
   aNum[2] = val>>8;
   aNum[3] = val;
-  sqlite3TestBinToHex(aNum, 4);
+  binToHex(aNum, 4);
   Tcl_SetObjResult(interp, Tcl_NewStringObj((char*)aNum, 8));
   return TCL_OK;
 }
 
-/*
-** USAGE:  utf8_to_utf8  HEX
-**
-** The argument is a UTF8 string represented in hexadecimal.
-** The UTF8 might not be well-formed.  Run this string through
-** sqlite3Utf8to8() convert it back to hex and return the result.
-*/
-static int utf8_to_utf8(
-  void * clientData,
-  Tcl_Interp *interp,
-  int objc,
-  Tcl_Obj *CONST objv[]
-){
-#ifdef SQLITE_DEBUG
-  int n;
-  int nOut;
-  const unsigned char *zOrig;
-  unsigned char *z;
-  if( objc!=2 ){
-    Tcl_WrongNumArgs(interp, 1, objv, "HEX");
-    return TCL_ERROR;
-  }
-  zOrig = (unsigned char *)Tcl_GetStringFromObj(objv[1], &n);
-  z = sqlite3_malloc( n+3 );
-  n = sqlite3TestHexToBin(zOrig, n, z);
-  z[n] = 0;
-  nOut = sqlite3Utf8To8(z);
-  sqlite3TestBinToHex(z,nOut);
-  Tcl_AppendResult(interp, (char*)z, 0);
-  sqlite3_free(z);
-  return TCL_OK;
-#else
-  Tcl_AppendResult(interp, 
-      "[utf8_to_utf8] unavailable - SQLITE_DEBUG not defined", 0
-  );
-  return TCL_ERROR;
-#endif
-}
-
-static int getFts3Varint(const char *p, sqlite_int64 *v){
-  const unsigned char *q = (const unsigned char *) p;
-  sqlite_uint64 x = 0, y = 1;
-  while( (*q & 0x80) == 0x80 ){
-    x += y * (*q++ & 0x7f);
-    y <<= 7;
-  }
-  x += y * (*q++);
-  *v = (sqlite_int64) x;
-  return (int) (q - (unsigned char *)p);
-}
-
-
-/*
-** USAGE:  read_fts3varint BLOB VARNAME
-**
-** Read a varint from the start of BLOB. Set variable VARNAME to contain
-** the interpreted value. Return the number of bytes of BLOB consumed.
-*/
-static int read_fts3varint(
-  void * clientData,
-  Tcl_Interp *interp,
-  int objc,
-  Tcl_Obj *CONST objv[]
-){
-  int nBlob;
-  unsigned char *zBlob;
-  sqlite3_int64 iVal;
-  int nVal;
-
-  if( objc!=3 ){
-    Tcl_WrongNumArgs(interp, 1, objv, "BLOB VARNAME");
-    return TCL_ERROR;
-  }
-  zBlob = Tcl_GetByteArrayFromObj(objv[1], &nBlob);
-
-  nVal = getFts3Varint((char*)zBlob, (sqlite3_int64 *)(&iVal));
-  Tcl_ObjSetVar2(interp, objv[2], 0, Tcl_NewWideIntObj(iVal), 0);
-  Tcl_SetObjResult(interp, Tcl_NewIntObj(nVal));
-  return TCL_OK;
-}
 
 
 /*
@@ -377,8 +302,6 @@ int Sqlitetest_hexio_Init(Tcl_Interp *interp){
      { "hexio_get_int",                hexio_get_int         },
      { "hexio_render_int16",           hexio_render_int16    },
      { "hexio_render_int32",           hexio_render_int32    },
-     { "utf8_to_utf8",                 utf8_to_utf8          },
-     { "read_fts3varint",              read_fts3varint       },
   };
   int i;
   for(i=0; i<sizeof(aObjCmd)/sizeof(aObjCmd[0]); i++){
