@@ -1,20 +1,64 @@
-set rcsid {$Id: capi3ref.tcl,v 1.39 2006/05/27 11:15:48 drh Exp $}
+set rcsid {$Id: capi3ref.tcl,v 1.53 2007/03/17 10:26:59 danielk1977 Exp $}
 source common.tcl
 header {C/C++ Interface For SQLite Version 3}
 puts {
-<h2>C/C++ Interface For SQLite Version 3</h2>
+<h2 class=pdf_section>C/C++ Interface For SQLite Version 3</h2>
 }
 
 proc api {name prototype desc {notused x}} {
-  global apilist
+  global apilist specialname
   if {$name==""} {
     regsub -all {sqlite3_[a-z0-9_]+\(} $prototype \
       {[lappend name [string trimright & (]]} x1
     subst $x1
+  } else {
+    lappend specialname $name
   }
   lappend apilist [list $name $prototype $desc]
 }
 
+api {extended-result-codes} {
+#define SQLITE_IOERR_READ       
+#define SQLITE_IOERR_SHORT_READ 
+#define SQLITE_IOERR_WRITE      
+#define SQLITE_IOERR_FSYNC      
+#define SQLITE_IOERR_DIR_FSYNC  
+#define SQLITE_IOERR_TRUNCATE   
+#define SQLITE_IOERR_FSTAT      
+#define SQLITE_IOERR_UNLOCK     
+#define SQLITE_IOERR_RDLOCK     
+...
+} {
+In its default configuration, SQLite API routines return one of 26 integer
+result codes described at result-codes.  However, experience has shown that
+many of these result codes are too course-grained.  They do not provide as
+much information about problems as users might like.  In an effort to
+address this, newer versions of SQLite (version 3.3.8 and later) include
+support for additional result codes that provide more detailed information
+about errors.  The extended result codes are enabled (or disabled) for 
+each database
+connection using the sqlite3_extended_result_codes() API.
+
+Some of the available extended result codes are listed above.
+We expect the number of extended result codes will be expand
+over time.  Software that uses extended result codes should expect
+to see new result codes in future releases of SQLite.
+
+The symbolic name for an extended result code always contains a related
+primary result code as a prefix.  Primary result codes contain a single
+"_" character.  Extended result codes contain two or more "_" characters.
+The numeric value of an extended result code can be converted to its
+corresponding primary result code by masking off the lower 8 bytes.
+
+A complete list of available extended result codes and
+details about the meaning of the various extended result codes can be
+found by consulting the C code, especially the sqlite3.h header
+file and its antecedent sqlite.h.in.  Additional information
+is also available at the SQLite wiki:
+http://www.sqlite.org/cvstrac/wiki?p=ExtendedResultCodes
+}
+
+
 api {result-codes} {
 #define SQLITE_OK           0   /* Successful result */
 #define SQLITE_ERROR        1   /* SQL error or missing database */
@@ -45,6 +89,27 @@ api {result-codes} {
 } {
 Many SQLite functions return an integer result code from the set shown
 above in order to indicates success or failure.
+
+The result codes above are the only ones returned by SQLite in its
+default configuration.  However, the sqlite3_extended_result_codes()
+API can be used to set a database connectoin to return more detailed
+result codes.  See the documentation on sqlite3_extended_result_codes()
+or extended-result-codes for additional information.
+}
+
+api {} {
+  int sqlite3_extended_result_codes(sqlite3*, int onoff);
+} {
+This routine enables or disabled extended-result-codes feature.
+By default, SQLite API routines return one of only 26 integer
+result codes described at result-codes.  When extended result codes
+are enabled by this routine, the repetoire of result codes can be
+much larger and can (hopefully) provide more detailed information
+about the cause of an error.
+
+The second argument is a boolean value that turns extended result
+codes on and off.  Extended result codes are off by default for
+backwards compatibility with older versions of SQLite.
 }
 
 api {} {
@@ -66,7 +131,8 @@ api {} {
   same aggregate instance) the same buffer is returned.  The implementation
   of the aggregate can use the returned buffer to accumulate data.
 
-  The buffer allocated is freed automatically by SQLite.
+  The buffer is freed automatically by SQLite when the query that
+  invoked the aggregate function terminates.
 }
 
 api {} {
@@ -91,15 +157,16 @@ api {} {
   #define SQLITE_STATIC      ((void(*)(void *))0)
   #define SQLITE_TRANSIENT   ((void(*)(void *))-1)
 } {
- In the SQL strings input to sqlite3_prepare() and sqlite3_prepare16(),
- one or more literals can be replace by a parameter "?" or ":AAA" or "\$VVV"
+ In the SQL strings input to sqlite3_prepare_v2() and sqlite3_prepare16_v2(),
+ one or more literals can be replace by a parameter "?" or ":AAA" or 
+ "@AAA" or "\$VVV"
  where AAA is an alphanumeric identifier and VVV is a variable name according
  to the syntax rules of the TCL programming language.
  The values of these parameters (also called "host parameter names")
  can be set using the sqlite3_bind_*() routines.
 
  The first argument to the sqlite3_bind_*() routines always is a pointer
- to the sqlite3_stmt structure returned from sqlite3_prepare().  The second
+ to the sqlite3_stmt structure returned from sqlite3_prepare_v2().  The second
  argument is the index of the parameter to be set.  The first parameter has
  an index of 1. When the same named parameter is used more than once, second
  and subsequent
@@ -123,10 +190,11 @@ api {} {
  special value SQLITE_STATIC, then the library assumes that the information
  is in static, unmanaged space and does not need to be freed.  If the
  fifth argument has the value SQLITE_TRANSIENT, then SQLite makes its
- own private copy of the data before returning.
+ own private copy of the data immediately, before the sqlite3_bind_*()
+ routine returns.
 
  The sqlite3_bind_*() routines must be called after
- sqlite3_prepare() or sqlite3_reset() and before sqlite3_step().
+ sqlite3_prepare_v2() or sqlite3_reset() and before sqlite3_step().
  Bindings are not cleared by the sqlite3_reset() routine.
  Unbound parameters are interpreted as NULL.
 
@@ -148,11 +216,13 @@ api {} {
   const char *sqlite3_bind_parameter_name(sqlite3_stmt*, int n);
 } {
   Return the name of the n-th parameter in the precompiled statement.
-  Parameters of the form ":AAA" or "\$VVV" have a name which is the
-  string ":AAA" or "\$VVV".  In other words, the initial ":" or "$"
+  Parameters of the form ":AAA" or "@AAA" or "\$VVV" have a name which is the
+  string ":AAA" or "\$VVV".  In other words, the initial ":" or "$" or "@"
   is included as part of the name.
   Parameters of the form "?" have no name.
 
+  The first bound parameter has an index of 1, not 0.
+
   If the value n is out of range or if the n-th parameter is nameless,
   then NULL is returned.  The returned string is always in the
   UTF-8 encoding.
@@ -177,8 +247,10 @@ api {} {
  upon encountering the lock.
  If the busy callback is not NULL, then the
  callback will be invoked with two arguments.  The
- second argument is the number of prior calls to the busy callback
- for the same lock.  If the
+ first argument to the handler is a copy of the void* pointer which
+ is the third argument to this routine.  The second argument to
+ the handler is the number of times that the busy handler has
+ been invoked for this locking event. If the
  busy callback returns 0, then no additional attempts are made to
  access the database and SQLITE_BUSY is returned.
  If the callback returns non-zero, then another attempt is made to open the
@@ -207,6 +279,11 @@ api {} {
  database.  Closing the database from a busy handler will delete 
  data structures out from under the executing query and will 
  probably result in a coredump.
+
+ There can only be a single busy handler defined for each database
+ connection.  Setting a new busy handler clears any previous one.
+ Note that calling sqlite3_busy_timeout() will also set or clear
+ the busy handler.
 }
 
 api {} {
@@ -220,6 +297,11 @@ api {} {
 
  Calling this routine with an argument less than or equal to zero
  turns off all busy handlers.
+
+ There can only be a single busy handler for a particular database
+ connection.  If another busy handler was defined  
+ (using sqlite3_busy_handler()) prior to calling
+ this routine, that other busy handler is cleared.
 }
 
 api {} {
@@ -301,7 +383,7 @@ int sqlite3_column_type(sqlite3_stmt*, int iCol);
  These routines return information about the information
  in a single column of the current result row of a query.  In every
  case the first argument is a pointer to the SQL statement that is being
- executed (the sqlite_stmt* that was returned from sqlite3_prepare()) and
+ executed (the sqlite_stmt* that was returned from sqlite3_prepare_v2()) and
  the second argument is the index of the column for which information 
  should be returned.  iCol is zero-indexed.  The left-most column has an
  index of 0.
@@ -549,6 +631,11 @@ int sqlite3_complete16(const void *sql);
  one or more complete SQL statements.
  The argument must be a nul-terminated UTF-8 string for sqlite3_complete()
  and a nul-terminated UTF-16 string for sqlite3_complete16().
+
+ These routines do not check to see if the SQL statement is well-formed.
+ They only check to see that the statement is terminated by a semicolon
+ that is not part of a string literal and is not inside
+ the body of a trigger.
 } {}
 
 api {} {
@@ -669,7 +756,11 @@ int sqlite3_create_function16(
  only difference between these two routines is that the second argument, the
  name of the (scalar) function or aggregate, is encoded in UTF-8 for
  sqlite3_create_function() and UTF-16 for sqlite3_create_function16().
-
+ The length of the name is limited to 255 bytes, exclusive of the 
+ zero-terminator.  Note that the name length limit is in bytes, not
+ characters.  Any attempt to create a function with a longer name
+ will result in an SQLITE_ERROR error.
+ 
  The first argument is the database handle that the new function or
  aggregate is to be added to. If a single program uses more than one
  database handle internally, then user functions or aggregates must 
@@ -678,7 +769,9 @@ int sqlite3_create_function16(
 
  The third argument is the number of arguments that the function or
  aggregate takes. If this argument is -1 then the function or
- aggregate may take any number of arguments.
+ aggregate may take any number of arguments.  The maximum number
+ of arguments to a new SQL function is 127.  A number larger than
+ 127 for the third argument results in an SQLITE_ERROR error.
 
  The fourth argument, eTextRep, specifies what type of text arguments
  this function prefers to receive.  Any function should be able to work
@@ -768,7 +861,7 @@ int sqlite3_exec(
  value then the query is aborted, all subsequent SQL statements
  are skipped and the sqlite3_exec() function returns the SQLITE_ABORT.
 
- The 4th argument is an arbitrary pointer that is passed
+ The 1st argument is an arbitrary pointer that is passed
  to the callback function as its first argument.
 
  The 2nd argument to the callback function is the number of
@@ -803,8 +896,9 @@ api {} {
 int sqlite3_finalize(sqlite3_stmt *pStmt);
 } {
  The sqlite3_finalize() function is called to delete a prepared
- SQL statement obtained by a previous call to sqlite3_prepare()
- or sqlite3_prepare16(). If the statement was executed successfully, or
+ SQL statement obtained by a previous call to sqlite3_prepare(),
+ sqlite3_prepare_v2(), sqlite3_prepare16(), or sqlite3_prepare16_v2().
+ If the statement was executed successfully, or
  not executed at all, then SQLITE_OK is returned. If execution of the
  statement failed then an error code is returned. 
 
@@ -820,10 +914,36 @@ int sqlite3_finalize(sqlite3_stmt *pStmt);
 }
 
 api {} {
-void sqlite3_free(char *z);
+void *sqlite3_malloc(int);
+void *sqlite3_realloc(void*, int);
+void sqlite3_free(void*);
 } {
- Use this routine to free memory obtained from 
- sqlite3_mprintf() or sqlite3_vmprintf().
+ These routines provide access to the memory allocator used by SQLite.
+ Depending on how SQLite has been compiled and the OS-layer backend,
+ the memory allocator used by SQLite might be the standard system
+ malloc()/realloc()/free(), or it might be something different.  With
+ certain compile-time flags, SQLite will add wrapper logic around the
+ memory allocator to add memory leak and buffer overrun detection.  The
+ OS layer might substitute a completely different memory allocator.
+ Use these APIs to be sure you are always using the correct memory
+ allocator.
+ 
+ The sqlite3_free() API, not the standard free() from the system library,
+ should always be used to free the memory buffer returned by
+ sqlite3_mprintf() or sqlite3_vmprintf() and to free the error message
+ string returned by sqlite3_exec().  Using free() instead of sqlite3_free()
+ might accidentally work on some systems and build configurations but 
+ will fail on others.
+
+ Compatibility Note:  Prior to version 3.4.0, the sqlite3_free API
+ was prototyped to take a <tt>char*</tt> parameter rather than 
+ <tt>void*</tt>.  Like this:
+<blockquote><pre>
+void sqlite3_free(char*);
+</pre></blockquote>
+ The change to using <tt>void*</tt> might cause warnings when 
+ compiling older code against
+ newer libraries, but everything should still work correctly.
 }
 
 api {} {
@@ -1006,6 +1126,22 @@ int sqlite3_open16(
 }
 
 api {} {
+int sqlite3_prepare_v2(
+  sqlite3 *db,            /* Database handle */
+  const char *zSql,       /* SQL statement, UTF-8 encoded */
+  int nBytes,             /* Length of zSql in bytes. */
+  sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
+  const char **pzTail     /* OUT: Pointer to unused portion of zSql */
+);
+int sqlite3_prepare16_v2(
+  sqlite3 *db,            /* Database handle */
+  const void *zSql,       /* SQL statement, UTF-16 encoded */
+  int nBytes,             /* Length of zSql in bytes. */
+  sqlite3_stmt **ppStmt,  /* OUT: Statement handle */
+  const void **pzTail     /* OUT: Pointer to unused portion of zSql */
+);
+
+/* Legacy Interfaces */
 int sqlite3_prepare(
   sqlite3 *db,            /* Database handle */
   const char *zSql,       /* SQL statement, UTF-8 encoded */
@@ -1022,14 +1158,13 @@ int sqlite3_prepare16(
 );
 } {
  To execute an SQL query, it must first be compiled into a byte-code
- program using one of the following routines. The only difference between
- them is that the second argument, specifying the SQL statement to
- compile, is assumed to be encoded in UTF-8 for the sqlite3_prepare()
- function and UTF-16 for sqlite3_prepare16().
+ program using one of these routines. 
 
  The first argument "db" is an SQLite database handle. The second
  argument "zSql" is the statement to be compiled, encoded as either
- UTF-8 or UTF-16 (see above). If the next argument, "nBytes", is less
+ UTF-8 or UTF-16.  The sqlite3_prepare_v2()
+ interfaces uses UTF-8 and sqlite3_prepare16_v2()
+ use UTF-16. If the next argument, "nBytes", is less
  than zero, then zSql is read up to the first nul terminator.  If
  "nBytes" is not less than zero, then it is the length of the string zSql
  in bytes (not characters).
@@ -1046,6 +1181,38 @@ int sqlite3_prepare16(
  using sqlite3_finalize() after it has finished with it.
 
  On success, SQLITE_OK is returned.  Otherwise an error code is returned.
+
+ The sqlite3_prepare_v2() and sqlite3_prepare16_v2() interfaces are
+ recommended for all new programs. The two older interfaces are retained
+ for backwards compatibility, but their use is discouraged.
+ In the "v2" interfaces, the prepared statement
+ that is returned (the sqlite3_stmt object) contains a copy of the original
+ SQL. This causes the sqlite3_step() interface to behave a differently in
+ two ways:
+
+ <ol>
+ <li>
+ If the database schema changes, instead of returning SQLITE_SCHEMA as it
+ always used to do, sqlite3_step() will automatically recompile the SQL
+ statement and try to run it again.  If the schema has changed in a way
+ that makes the statement no longer valid, sqlite3_step() will still
+ return SQLITE_SCHEMA.  But unlike the legacy behavior, SQLITE_SCHEMA is
+ now a fatal error.  Calling sqlite3_prepare_v2() again will not make the
+ error go away.  Note: use sqlite3_errmsg() to find the text of the parsing
+ error that results in an SQLITE_SCHEMA return.
+ </li>
+
+ <li>
+ When an error occurs, 
+ sqlite3_step() will return one of the detailed result-codes
+ like SQLITE_IOERR or SQLITE_FULL or SQLITE_SCHEMA directly. The
+ legacy behavior was that sqlite3_step() would only return a generic
+ SQLITE_ERROR code and you would have to make a second call to
+ sqlite3_reset() in order to find the underlying cause of the problem.
+ With the "v2" prepare interfaces, the underlying reason for the error is
+ returned directly.
+ </li>
+ </ol>
 }
 
 api {} {
@@ -1083,8 +1250,9 @@ api {} {
 int sqlite3_reset(sqlite3_stmt *pStmt);
 } {
  The sqlite3_reset() function is called to reset a prepared SQL
- statement obtained by a previous call to sqlite3_prepare() or
- sqlite3_prepare16() back to it's initial state, ready to be re-executed.
+ statement obtained by a previous call to 
+ sqlite3_prepare_v2() or
+ sqlite3_prepare16_v2() back to it's initial state, ready to be re-executed.
  Any SQL statement variables that had values bound to them using
  the sqlite3_bind_*() API retain their values.
 }
@@ -1146,45 +1314,85 @@ int sqlite3_set_authorizer(
 #define SQLITE_ALTER_TABLE          26   /* Database Name   Table Name      */
 #define SQLITE_REINDEX              27   /* Index Name      NULL            */
 #define SQLITE_ANALYZE              28   /* Table Name      NULL            */
+#define SQLITE_CREATE_VTABLE        29   /* Table Name      Module Name     */
+#define SQLITE_DROP_VTABLE          30   /* Table Name      Module Name     */
+#define SQLITE_FUNCTION             31   /* Function Name   NULL            */
 
 #define SQLITE_DENY   1   /* Abort the SQL statement with an error */
 #define SQLITE_IGNORE 2   /* Don't allow access, but don't generate an error */
 } {
  This routine registers a callback with the SQLite library.  The
- callback is invoked (at compile-time, not at run-time) for each
- attempt to access a column of a table in the database.  The callback should
+ callback is invoked by sqlite3_prepare_v2() to authorize various
+ operations against the database.  The callback should
  return SQLITE_OK if access is allowed, SQLITE_DENY if the entire
  SQL statement should be aborted with an error and SQLITE_IGNORE
- if the column should be treated as a NULL value.
+ if the operation should be treated as a no-op.
+
+ Each database connection have at most one authorizer registered
+ at a time one time.  Each call
+ to sqlite3_set_authorizer() overrides the previous authorizer.
+ Setting the callback to NULL disables the authorizer.
 
  The second argument to the access authorization function will be one
  of the defined constants shown.  These values signify what kind of operation
  is to be authorized.  The 3rd and 4th arguments to the authorization
- function will be arguments or NULL depending on which of the following
- codes is used as the second argument.  The 5th argument is the name
- of the database ("main", "temp", etc.) if applicable.  The 6th argument
+ function will be arguments or NULL depending on which of the 
+ codes is used as the second argument.  For example, if the the
+ 2nd argument code is SQLITE_READ then the 3rd argument will be the name
+ of the table that is being read from and the 4th argument will be the
+ name of the column that is being read from.  Or if the 2nd argument
+ is SQLITE_FUNCTION then the 3rd argument will be the name of the
+ function that is being invoked and the 4th argument will be NULL.
+
+ The 5th argument is the name
+ of the database ("main", "temp", etc.) where applicable.  The 6th argument
  is the name of the inner-most trigger or view that is responsible for
  the access attempt or NULL if this access attempt is directly from 
  input SQL code.
 
- The return value of the authorization function should be one of the
- constants SQLITE_OK, SQLITE_DENY, or SQLITE_IGNORE.
+ The return value of the authorization callback function should be one of the
+ constants SQLITE_OK, SQLITE_DENY, or SQLITE_IGNORE.  A return of
+ SQLITE_OK means that the operation is permitted and that 
+ sqlite3_prepare_v2() can proceed as normal.
+ A return of SQLITE_DENY means that the sqlite3_prepare_v2()
+ should fail with an error.  A return of SQLITE_IGNORE causes the 
+ sqlite3_prepare_v2() to continue as normal but the requested 
+ operation is silently converted into a no-op.  A return of SQLITE_IGNORE
+ in response to an SQLITE_READ or SQLITE_FUNCTION causes the column
+ being read or the function being invoked to return a NULL.
 
  The intent of this routine is to allow applications to safely execute
  user-entered SQL.  An appropriate callback can deny the user-entered
  SQL access certain operations (ex: anything that changes the database)
  or to deny access to certain tables or columns within the database.
+
+ SQLite is not reentrant through the authorization callback function.
+ The authorization callback function should not attempt to invoke
+ any other SQLite APIs for the same database connection.  If the
+ authorization callback function invokes some other SQLite API, an
+ SQLITE_MISUSE error or a segmentation fault may result.
 }
 
 api {} {
 int sqlite3_step(sqlite3_stmt*);
 } {
  After an SQL query has been prepared with a call to either
- sqlite3_prepare() or sqlite3_prepare16(), then this function must be
+ sqlite3_prepare_v2() or sqlite3_prepare16_v2() or to one of
+ the legacy interfaces sqlite3_prepare() or sqlite3_prepare16(),
+ then this function must be
  called one or more times to execute the statement.
 
- The return value will be either SQLITE_BUSY, SQLITE_DONE, 
- SQLITE_ROW, SQLITE_ERROR, or SQLITE_MISUSE.
+ The details of the behavior of this sqlite3_step() interface depend
+ on whether the statement was prepared using the newer "v2" interface
+ sqlite3_prepare_v2() and sqlite3_prepare16_v2() or the older legacy
+ interface sqlite3_prepare() and sqlite3_prepare16().  The use of the
+ new "v2" interface is recommended for new applications but the legacy
+ interface will continue to be supported.
+
+ In the lagacy interface, the return value will be either SQLITE_BUSY, 
+ SQLITE_DONE, SQLITE_ROW, SQLITE_ERROR, or SQLITE_MISUSE.  With the "v2"
+ interface, any of the other SQLite result-codes might be returned as
+ well.
 
  SQLITE_BUSY means that the database engine attempted to open
  a locked database and there is no busy callback registered.
@@ -1198,18 +1406,35 @@ int sqlite3_step(sqlite3_stmt*);
  If the SQL statement being executed returns any data, then 
  SQLITE_ROW is returned each time a new row of data is ready
  for processing by the caller. The values may be accessed using
- the sqlite3_column_*() functions. sqlite3_step()
- is called again to retrieve the next row of data.
+ the sqlite3_column_int(), sqlite3_column_text(), and similar functions.
+ sqlite3_step() is called again to retrieve the next row of data.
  
  SQLITE_ERROR means that a run-time error (such as a constraint
  violation) has occurred.  sqlite3_step() should not be called again on
  the VM. More information may be found by calling sqlite3_errmsg().
+ A more specific error code (example: SQLITE_INTERRUPT, SQLITE_SCHEMA,
+ SQLITE_CORRUPT, and so forth) can be obtained by calling
+ sqlite3_reset() on the prepared statement.  In the "v2" interface,
+ the more specific error code is returned directly by sqlite3_step().
 
  SQLITE_MISUSE means that the this routine was called inappropriately.
  Perhaps it was called on a virtual machine that had already been
  finalized or on one that had previously returned SQLITE_ERROR or
  SQLITE_DONE.  Or it could be the case that a database connection
  is being used by a different thread than the one it was created it.
+
+ <b>Goofy Interface Alert:</b>
+ In the legacy interface, 
+ the sqlite3_step() API always returns a generic error code,
+ SQLITE_ERROR, following any error other than SQLITE_BUSY and SQLITE_MISUSE.
+ You must call sqlite3_reset() (or sqlite3_finalize()) in order to find
+ one of the specific result-codes that better describes the error.
+ We admit that this is a goofy design.  The problem has been fixed
+ with the "v2" interface.  If you prepare all of your SQL statements
+ using either sqlite3_prepare_v2() or sqlite3_prepare16_v2() instead
+ of the legacy sqlite3_prepare() and sqlite3_prepare16(), then the 
+ more specific result-codes are returned directly by sqlite3_step().
+ The use of the "v2" interface is recommended.
 }
 
 api {} {
@@ -1217,7 +1442,7 @@ void *sqlite3_trace(sqlite3*, void(*xTrace)(void*,const char*), void*);
 } {
  Register a function that is called each time an SQL statement is evaluated.
  The callback function is invoked on the first call to sqlite3_step() after
- calls to sqlite3_prepare() or sqlite3_reset().
+ calls to sqlite3_prepare_v2() or sqlite3_reset().
  This function can be used (for example) to generate
  a log file of all SQL executed against a database.  This can be
  useful when debugging an application that uses SQLite.
@@ -1396,7 +1621,7 @@ api {} {
 
   When the shared cache is enabled, the
   following routines must always be called from the same thread:
-  sqlite3_open(), sqlite3_prepare(), sqlite3_step(), sqlite3_reset(),
+  sqlite3_open(), sqlite3_prepare_v2(), sqlite3_step(), sqlite3_reset(),
   sqlite3_finalize(), and sqlite3_close().
   This is due to the fact that the shared cache makes use of
   thread-specific storage so that it will be available for sharing
@@ -1497,6 +1722,7 @@ foreach name [lsort [array names name_to_idx]] {
 #parray name_to_idx
 #parray sname
 incr n -1
+puts "<DIV class=pdf_ignore>"
 puts {<table width="100%" cellpadding="5"><tr>}
 set nrow [expr {($n+2)/3}]
 set i 0
@@ -1515,6 +1741,7 @@ for {set j 0} {$j<3} {incr j} {
 }
 puts "</table>"
 puts "<!-- $n entries.  $nrow rows in 3 columns -->"
+puts "</DIV>"
 
 proc resolve_name {ignore_list name} {
   global name_to_idx
@@ -1531,7 +1758,7 @@ foreach name [lsort [array names name_to_idx]] {
   set done($i) 1
   foreach {namelist prototype desc} [lindex $apilist $i] break
   foreach name $namelist {
-    puts "<a name=\"$name\">"
+    puts "<a name=\"$name\"></a>"
   }
   puts "<p><hr></p>"
   puts "<blockquote><pre>"
@@ -1541,8 +1768,13 @@ foreach name [lsort [array names name_to_idx]] {
   puts "</pre></blockquote>"
   regsub -all {\[} $desc {\[} desc
   regsub -all {sqlite3_[a-z0-9_]+} $desc "\[resolve_name $name &\]" d2
+  foreach x $specialname {
+    regsub -all $x $d2 "\[resolve_name $name &\]" d2
+  }
   regsub -all "\n( *\n)+" [subst $d2] "</p>\n\n<p>" d3
   puts "<p>$d3</p>"
 }
 
+puts "<DIV class=pdf_ignore>"
 footer $rcsid
+puts "</DIV>"
